@@ -5,14 +5,16 @@ Clustering package (:mod:`scipy.cluster`)
 
 .. currentmodule:: scipy.cluster
 
-:mod:`scipy.cluster.vq`
+.. toctree::
+   :hidden:
+
+   cluster.vq
+   cluster.hierarchy
 
 Clustering algorithms are useful in information theory, target detection,
-communications, compression, and other areas.  The `vq` module only
+communications, compression, and other areas. The `vq` module only
 supports vector quantization and the k-means algorithms.
 
-:mod:`scipy.cluster.hierarchy`
-
 The `hierarchy` module provides functions for hierarchical and
 agglomerative clustering.  Its features include generating hierarchical
 clusters from distance matrices,
@@ -20,8 +22,6 @@ calculating statistics on clusters, cutting linkages
 to generate flat clusters, and visualizing clusters with dendrograms.
 
 """
-from __future__ import division, print_function, absolute_import
-
 __all__ = ['vq', 'hierarchy']
 
 from . import vq, hierarchy
