@@ -6,10 +6,10 @@ from numpy.testing import (assert_, assert_equal, assert_array_almost_equal,
 import pytest
 from pytest import raises as assert_raises
 from scipy.fft._pocketfft import (ifft, fft, fftn, ifftn,
-                                  rfft, irfft, rfftn, irfftn, fft2,
+                                  rfft, irfft, rfftn, irfftn,
                                   hfft, ihfft, hfftn, ihfftn)
 
-from numpy import (arange, add, array, asarray, zeros, dot, exp, pi,
+from numpy import (arange, array, asarray, zeros, dot, exp, pi,
                    swapaxes, cdouble)
 import numpy as np
 import numpy.fft
@@ -37,7 +37,7 @@ SMALL_PRIME_SIZES = [
 
 def _assert_close_in_norm(x, y, rtol, size, rdt):
     # helper function for testing
-    err_msg = "size: %s  rdt: %s" % (size, rdt)
+    err_msg = f"size: {size}  rdt: {rdt}"
     assert_array_less(np.linalg.norm(x - y), rtol*np.linalg.norm(x), err_msg)
 
 
@@ -49,12 +49,6 @@ def swap_byteorder(arr):
     dtype = arr.dtype.newbyteorder('S')
     return arr.astype(dtype)
 
-def get_mat(n):
-    data = arange(n)
-    data = add.outer(data, data)
-    return data
-
-
 def direct_dft(x):
     x = asarray(x)
     n = len(x)
@@ -163,14 +157,14 @@ class _TestFFTBase:
 
 class TestLongDoubleFFT(_TestFFTBase):
     def setup_method(self):
-        self.cdt = np.longcomplex
+        self.cdt = np.clongdouble
         self.rdt = np.longdouble
 
 
 class TestDoubleFFT(_TestFFTBase):
     def setup_method(self):
         self.cdt = np.cdouble
-        self.rdt = np.double
+        self.rdt = np.float64
 
 
 class TestSingleFFT(_TestFFTBase):
@@ -279,7 +273,7 @@ class _TestIFFTBase:
                     reason="Long double is aliased to double")
 class TestLongDoubleIFFT(_TestIFFTBase):
     def setup_method(self):
-        self.cdt = np.longcomplex
+        self.cdt = np.clongdouble
         self.rdt = np.longdouble
         self.rtol = 1e-10
         self.atol = 1e-10
@@ -287,8 +281,8 @@ class TestLongDoubleIFFT(_TestIFFTBase):
 
 class TestDoubleIFFT(_TestIFFTBase):
     def setup_method(self):
-        self.cdt = np.cdouble
-        self.rdt = np.double
+        self.cdt = np.complex128
+        self.rdt = np.float64
         self.rtol = 1e-10
         self.atol = 1e-10
 
@@ -339,9 +333,8 @@ class _TestRFFTBase:
             try:
                 return getattr(self.data, item)
             except AttributeError as e:
-                raise AttributeError(("'MockSeries' object "
-                                      "has no attribute '{attr}'".
-                                      format(attr=item))) from e
+                raise AttributeError("'MockSeries' object "
+                                      f"has no attribute '{item}'") from e
 
     def test_non_ndarray_with_dtype(self):
         x = np.array([1., 2., 3., 4., 5.])
@@ -354,18 +347,18 @@ class _TestRFFTBase:
         assert_equal(x, expected)
         assert_equal(xs.data, expected)
 
-@pytest.mark.skipif(np.longfloat is np.float64,
+@pytest.mark.skipif(np.longdouble is np.float64,
                     reason="Long double is aliased to double")
 class TestRFFTLongDouble(_TestRFFTBase):
     def setup_method(self):
-        self.cdt = np.longcomplex
-        self.rdt = np.longfloat
+        self.cdt = np.clongdouble
+        self.rdt = np.longdouble
 
 
 class TestRFFTDouble(_TestRFFTBase):
     def setup_method(self):
-        self.cdt = np.cdouble
-        self.rdt = np.double
+        self.cdt = np.complex128
+        self.rdt = np.float64
 
 
 class TestRFFTSingle(_TestRFFTBase):
@@ -441,19 +434,19 @@ class _TestIRFFTBase:
 # self.ndec is bogus; we should have a assert_array_approx_equal for number of
 # significant digits
 
-@pytest.mark.skipif(np.longfloat is np.float64,
+@pytest.mark.skipif(np.longdouble is np.float64,
                     reason="Long double is aliased to double")
 class TestIRFFTLongDouble(_TestIRFFTBase):
     def setup_method(self):
-        self.cdt = np.cdouble
-        self.rdt = np.double
+        self.cdt = np.complex128
+        self.rdt = np.float64
         self.ndec = 14
 
 
 class TestIRFFTDouble(_TestIRFFTBase):
     def setup_method(self):
-        self.cdt = np.cdouble
-        self.rdt = np.double
+        self.cdt = np.complex128
+        self.rdt = np.float64
         self.ndec = 14
 
 
@@ -464,24 +457,6 @@ class TestIRFFTSingle(_TestIRFFTBase):
         self.ndec = 5
 
 
-class Testfft2:
-    def setup_method(self):
-        np.random.seed(1234)
-
-    def test_regression_244(self):
-        """FFT returns wrong result with axes parameter."""
-        # fftn (and hence fft2) used to break when both axes and shape were
-        # used
-        x = numpy.ones((4, 4, 2))
-        y = fft2(x, s=(8, 8), axes=(-3, -2))
-        y_r = numpy.fft.fftn(x, s=(8, 8), axes=(-3, -2))
-        assert_array_almost_equal(y, y_r)
-
-    def test_invalid_sizes(self):
-        assert_raises(ValueError, fft2, [[]])
-        assert_raises(ValueError, fft2, [[1, 1], [2, 2]], (4, -3))
-
-
 class TestFftnSingle:
     def setup_method(self):
         np.random.seed(1234)
@@ -761,6 +736,14 @@ class TestFftn:
         x = numpy.random.random((2,2,2))
         assert_allclose(fftn(x, axes=[]), x, atol=1e-7)
 
+    def test_regression_244(self):
+        """FFT returns wrong result with axes parameter."""
+        # fftn (and hence fft2) used to break when both axes and shape were used
+        x = numpy.ones((4, 4, 2))
+        y = fftn(x, s=(8, 8), axes=(-3, -2))
+        y_r = numpy.fft.fftn(x, s=(8, 8), axes=(-3, -2))
+        assert_allclose(y, y_r)
+
 
 class TestIfftn:
     dtype = None
@@ -878,8 +861,8 @@ class FakeArray2:
 class TestOverwrite:
     """Check input overwrite behavior of the FFT functions."""
 
-    real_dtypes = [np.float32, np.float64, np.longfloat]
-    dtypes = real_dtypes + [np.complex64, np.complex128, np.longcomplex]
+    real_dtypes = [np.float32, np.float64, np.longdouble]
+    dtypes = real_dtypes + [np.complex64, np.complex128, np.clongdouble]
     fftsizes = [8, 16, 32]
 
     def _check(self, x, routine, fftsize, axis, overwrite_x, should_overwrite):
@@ -887,7 +870,7 @@ class TestOverwrite:
         for fake in [lambda x: x, FakeArray, FakeArray2]:
             routine(fake(x2), fftsize, axis, overwrite_x=overwrite_x)
 
-            sig = "%s(%s%r, %r, axis=%r, overwrite_x=%r)" % (
+            sig = "{}({}{!r}, {!r}, axis={!r}, overwrite_x={!r})".format(
                 routine.__name__, x.dtype, x.shape, fftsize, axis, overwrite_x)
             if not should_overwrite:
                 assert_equal(x2, x, err_msg="spurious overwrite in %s" % sig)
@@ -915,7 +898,7 @@ class TestOverwrite:
                                             ((16, 2), 0),
                                             ((2, 16), 1)])
     def test_fft_ifft(self, dtype, fftsize, overwrite_x, shape, axes):
-        overwritable = (np.longcomplex, np.complex128, np.complex64)
+        overwritable = (np.clongdouble, np.complex128, np.complex64)
         self._check_1d(fft, dtype, shape, axes, overwritable,
                        fftsize, overwrite_x)
         self._check_1d(ifft, dtype, shape, axes, overwritable,
@@ -990,7 +973,7 @@ class TestOverwrite:
                                             ((8, 16, 2), None),
                                             ((8, 16, 2), (0, 1, 2))])
     def test_fftn_ifftn(self, dtype, overwrite_x, shape, axes):
-        overwritable = (np.longcomplex, np.complex128, np.complex64)
+        overwritable = (np.clongdouble, np.complex128, np.complex64)
         self._check_nd_one(fftn, dtype, shape, axes, overwritable,
                            overwrite_x)
         self._check_nd_one(ifftn, dtype, shape, axes, overwritable,
