@@ -28,12 +28,11 @@
 # NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 # SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 
-from __future__ import division, print_function, absolute_import
-
+from collections.abc import Iterable
+import operator
+import warnings
 import numpy
 
-from scipy._lib.six import string_types
-
 
 def _extend_mode_to_code(mode):
     """Convert an extension mode to the corresponding integer code.
@@ -42,12 +41,16 @@ def _extend_mode_to_code(mode):
         return 0
     elif mode == 'wrap':
         return 1
-    elif mode == 'reflect':
+    elif mode in ['reflect', 'grid-mirror']:
         return 2
     elif mode == 'mirror':
         return 3
     elif mode == 'constant':
         return 4
+    elif mode == 'grid-wrap':
+        return 5
+    elif mode == 'grid-constant':
+        return 6
     else:
         raise RuntimeError('boundary mode not supported')
 
@@ -57,8 +60,8 @@ def _normalize_sequence(input, rank):
     rank by duplicating the input. If input is a sequence,
     check if its length is equal to the length of array.
     """
-    is_str = isinstance(input, string_types)
-    if hasattr(input, '__iter__') and not is_str:
+    is_str = isinstance(input, str)
+    if not is_str and isinstance(input, Iterable):
         normalized = list(input)
         if len(normalized) != rank:
             err = "sequence argument must have length equal to input rank"
@@ -68,25 +71,53 @@ def _normalize_sequence(input, rank):
     return normalized
 
 
-def _get_output(output, input, shape=None):
+def _get_output(output, input, shape=None, complex_output=False):
     if shape is None:
         shape = input.shape
     if output is None:
-        output = numpy.zeros(shape, dtype=input.dtype.name)
+        if not complex_output:
+            output = numpy.zeros(shape, dtype=input.dtype.name)
+        else:
+            complex_type = numpy.promote_types(input.dtype, numpy.complex64)
+            output = numpy.zeros(shape, dtype=complex_type)
     elif isinstance(output, (type, numpy.dtype)):
         # Classes (like `np.float32`) and dtypes are interpreted as dtype
+        if complex_output and numpy.dtype(output).kind != 'c':
+            warnings.warn("promoting specified output dtype to complex", stacklevel=3)
+            output = numpy.promote_types(output, numpy.complex64)
         output = numpy.zeros(shape, dtype=output)
-    elif isinstance(output, string_types):
-        output = numpy.typeDict[output]
+    elif isinstance(output, str):
+        # testsuite only appears to cover
+        # f->np.float32 here
+        f_dict = {"f": numpy.float32,
+                  "d": numpy.float64,
+                  "F": numpy.complex64,
+                  "D": numpy.complex128}
+        output = f_dict[output]
+        if complex_output and numpy.dtype(output).kind != 'c':
+            raise RuntimeError("output must have complex dtype")
         output = numpy.zeros(shape, dtype=output)
     elif output.shape != shape:
         raise RuntimeError("output shape not correct")
+    elif complex_output and output.dtype.kind != 'c':
+        raise RuntimeError("output must have complex dtype")
     return output
 
 
-def _check_axis(axis, rank):
-    if axis < 0:
-        axis += rank
-    if axis < 0 or axis >= rank:
-        raise ValueError('invalid axis')
-    return axis
+def _check_axes(axes, ndim):
+    if axes is None:
+        return tuple(range(ndim))
+    elif numpy.isscalar(axes):
+        axes = (operator.index(axes),)
+    elif isinstance(axes, Iterable):
+        for ax in axes:
+            axes = tuple(operator.index(ax) for ax in axes)
+            if ax < -ndim or ax > ndim - 1:
+                raise ValueError(f"specified axis: {ax} is out of range")
+        axes = tuple(ax % ndim if ax < 0 else ax for ax in axes)
+    else:
+        message = "axes must be an integer, iterable of integers, or None"
+        raise ValueError(message)
+    if len(tuple(set(axes))) != len(axes):
+        raise ValueError("axes must be unique")
+    return axes
